﻿//	Transform Tools Script
//
//	Freeze\Reset Transform functionality similar to that of Maya.
//	Freeze the selection before editing any of the object's transforms
//	and restore the position with Reset. Only saves last Freeze Transform.
//	
//	Parent to empty GameObject functionality allows you to quickly group items
//	under organized parent objects. The parent object is selected after grouping.
//
//	Code by Doug Clayton
//	Feel free to use or edit.

using UnityEngine;
using UnityEditor;
using System.Collections;

public class TransformTools : MonoBehaviour
{
	public struct SavedTransform
	{
		public Vector3 position;
		public Quaternion rotation;
		public Vector3 scale;
		public bool transformSaved;
		public int transformID;
	}
	
	static SavedTransform saveTran;	
	static SavedTransform[] savedTrans;
	
	//Freezes Transforms
	[MenuItem("Tools/Transform/Freeze Transform")]
	public static void FreezeTransform()
	{
		//Single objects
		if(Selection.transforms.Length == 1)
		{			
			saveTran = new SavedTransform();
			
			saveTran.position = Selection.activeTransform.localPosition;
			saveTran.rotation = Selection.activeTransform.localRotation;
			saveTran.scale = Selection.activeTransform.localScale;
			saveTran.transformSaved = true;
			saveTran.transformID = Selection.activeGameObject.GetInstanceID();
		}
		
		//Multiple objects
		else if(Selection.transforms.Length >= 2)
		{			
			savedTrans = new SavedTransform[Selection.transforms.Length];
			
			for(int i = 0; i < Selection.transforms.Length; i++)
			{			
				savedTrans[i].position = Selection.transforms[i].localPosition;
				savedTrans[i].rotation = Selection.transforms[i].localRotation;
				savedTrans[i].scale = Selection.transforms[i].localScale;
				savedTrans[i].transformSaved = true;
				savedTrans[i].transformID = Selection.gameObjects[i].GetInstanceID();
			}			
		}		
	}
	
	//Resets Transformations
	[MenuItem("Tools/Transform/Reset Transform")]
	public static void ResetTransform()
	{	
		//Single objects
		if(Selection.transforms.Length == 1)
		{
			if(saveTran.transformSaved == true && saveTran.transformID == Selection.activeGameObject.GetInstanceID())
			{
				Selection.activeTransform.localPosition = saveTran.position;
				Selection.activeTransform.localRotation = saveTran.rotation;
				Selection.activeTransform.localScale = saveTran.scale;
			}
			
			else
			{
				Debug.Log("No Transform was frozen for " + Selection.activeGameObject.name);
			}
		}		
		
		//Multiple objects
		else if(Selection.transforms.Length >= 2)
		{
			for(int i = 0; i < Selection.transforms.Length; i++)
			{	
				if(savedTrans[i].transformSaved == true && savedTrans[i].transformID == Selection.gameObjects[i].GetInstanceID())
				{
					Selection.transforms[i].localPosition = savedTrans[i].position;
					Selection.transforms[i].localRotation = savedTrans[i].rotation;
					Selection.transforms[i].localScale = savedTrans[i].scale;
				}
				
				else
				{
					Debug.Log("No Transform was frozen for " + Selection.transforms[i].gameObject.name);
				}
			}
		}
	}
	
	//Parents selected objects to an empty GameObject (Centered pivot)
	[MenuItem("Tools/Transform/Parent Objects To Empty GameObject (Centered)")]
	public static void ParentToEmptyGOCenter()
	{		
		GameObject[] gos = Selection.gameObjects;
		GameObject newParent = new GameObject("_GameObject");
		Vector3 centerPos = new Vector3(0.0f,0.0f,0.0f);
		
		Undo.RegisterSceneUndo("Parent Empty GameObject");
		
		//Math to get central point between all transforms.
		foreach(GameObject go in gos)
		{
			centerPos += go.transform.localPosition;
		}
		
		newParent.transform.position = centerPos / (gos.Length);
		
		foreach(GameObject go in gos)
		{			
			Undo.RegisterUndo(go, "Parent Empty GameObject (Center)");
			go.transform.parent = newParent.transform;
		}
		
		Selection.activeGameObject = newParent;
	}
	
	//Parents selected objects to an empty GameObject (Pivot at origin)
	[MenuItem("Tools/Transform/Parent Objects To Empty GameObject (Origin)")]
	public static void ParentToEmptyGOZeroed()
	{
		GameObject[] gos = Selection.gameObjects;
		GameObject newParent = new GameObject("_GameObject");		
		
		
		newParent.transform.localPosition = Vector3.zero;
		
		foreach(GameObject go in gos)
		{
			Undo.RegisterUndo(go, "Parent Empty GameObject (Origin)");
			go.transform.parent = newParent.transform;
		}
		
		Selection.activeGameObject = newParent;
	}
}
